/*
 * calc pi by Monte Carlo simulation.
 */
#include <stdio.h>
#include <stdlib.h> /* for random */
#include <math.h> /* for fabs and M_PI */
#include "mpi.h"

#define N_POINTS 100000000

int main(int argc,char *argv[])
{
    int    myid, n_procs;
    int    namelen;
    char   processor_name[MPI_MAX_PROCESSOR_NAME];
    double startwtime = 0.0, endwtime;
    int n, incnt, intotal, i;
    double x, y, dist, pi;

    MPI_Init(&argc,&argv);
    MPI_Comm_size(MPI_COMM_WORLD,&n_procs);
    MPI_Comm_rank(MPI_COMM_WORLD,&myid);
    MPI_Get_processor_name(processor_name,&namelen);

    fprintf(stdout,"Process %d of %d is on %s\n",
	    myid, n_procs, processor_name);

    if (myid == 0) {
	startwtime = MPI_Wtime();
	n = N_POINTS / n_procs;
    }

    MPI_Bcast(&n, 1, MPI_INT, 0, MPI_COMM_WORLD);

    srandom(0);
    incnt = 0;
    for (i = 0; i < n; i++) {
	x = (double)random() / RAND_MAX;
	y = (double)random() / RAND_MAX;
	dist = sqrt(x*x + y*y);
	if (dist < 1) incnt +=1;
    }

    MPI_Reduce(&incnt, &intotal, 1, MPI_INT, MPI_SUM, 0, MPI_COMM_WORLD);

    if (myid == 0) {
	pi = 4 * (double)intotal / N_POINTS;
	endwtime = MPI_Wtime();
	printf("pi is approximately %.16f, Error is %.16f\n",
	       pi, fabs(pi - M_PI));
	printf("wall clock time = %f\n", endwtime-startwtime);	       
    }

    MPI_Finalize();
    return 0;
}
